﻿<%@ WebHandler Language="C#" Debug="true" Class="dms" %>

using System;
using System.Web;

//TODO: add async functionality
public class dms : IHttpHandler
{
    /// <summary>
    /// Process the incoming request to upload a file.
    /// </summary>
    /// <param name="context"></param>
    public void ProcessRequest(HttpContext context)
    {
        context.Response.ContentType = "text/plain";
       //new object to house status, messages, and link
        DMSMultiUpload upload = new DMSMultiUpload();
        upload.success = false;
        try
        {
            //check for a valid folder id first
            if (!string.IsNullOrEmpty(context.Request.Form["fid"]) && context.Request.Form["fid"] != "undefined")
            {
                //check if a dms (0) or library (1) type are specified
                if (!string.IsNullOrEmpty(context.Request.Form["dmstype"]))
                {
                    string dmstype = context.Request.Form["dmstype"];
                    if (dmstype == "0")
                    {
                        //check permissions for DMS
                        if (PermissionsCheck(context))
                        {
                            //upload file(s)
                            upload = ReadItems(context);
                        }
                        else
                        {
                            //no permissions
                            upload.Message = "Invalid permissions/folder access";
                        }
                    }
                    else if (dmstype == "1")
                    {
                        //check permissions for library
                        if (PermissionsCheck(context, false))
                        {
                            //upload file(s)
                            upload = ReadItems(context, false);
                        }
                        else
                        {
                            //no permissions
                            upload.Message = "Invalid permissions/folder access";
                        }
                    }
                    else
                    {
                        //no valid dms type
                        upload.Message = "Invalid dmstype specified";
                    }
                }
            }
            else
            {
                //No folder to upload to was specified
                upload.Message = "No upload folder selected.";
            }
        }
        catch (Exception ex)
        {
            upload.Message = "An exception occurred. Please contact your system administrator.";
        }
        //return json response back to the dropzonejs function
        System.Web.Script.Serialization.JavaScriptSerializer jss = new System.Web.Script.Serialization.JavaScriptSerializer();
        context.Response.Write(jss.Serialize(upload));
    }

    public bool IsReusable
    {
        get
        {
            return false;
        }
    }

    /// <summary>
    /// Loop through the files to be uploaded and add to the DMS or Library
    /// </summary>
    /// <param name="context">Current HTTP Context</param>
    /// <param name="dms">DMS = true, Library = false</param>
    /// <returns>DMSMultiUpload containing status, message, and link</returns>
    private DMSMultiUpload ReadItems(HttpContext context, bool dms = true)
    {
        DMSMultiUpload multiupload = new DMSMultiUpload();
        multiupload.success = false;
        string status = "";
        //loop through files uploaded in the context request. Currently the control is set to upload one at a time though
        HttpFileCollection fileCollection = context.Request.Files;
        for (int i = 0; i < fileCollection.Count; i++)
        {
            try
            {
                bool uploadSuccess = false;
                //write file to a byte array to upload to the cms
                HttpPostedFile upload = fileCollection[i];
                int fileLength = upload.ContentLength;
                byte[] fileData = new byte[fileLength];
                upload.InputStream.Read(fileData, 0, fileLength);
                if (dms)
                {
                    //uploading DMS documents
                    Ektron.Cms.Framework.Content.AssetManager assetManager = new Ektron.Cms.Framework.Content.AssetManager();
                    //populate asset data
                    Ektron.Cms.Content.ContentAssetData contentAssetData = new Ektron.Cms.Content.ContentAssetData()
                    {
                        FolderId = long.Parse(context.Request.Form["fid"]),
                        Title = fileCollection[i].FileName.ToString().Substring(0, fileCollection[i].FileName.IndexOf('.')),
                        Teaser = "",
                        File = fileData,
                        LanguageId = assetManager.RequestInformation.ContentLanguage,
                        AssetData = new Ektron.Cms.Common.AssetData()
                        {
                            FileName = fileCollection[i].FileName
                        }
                    };
                    //add the assset
                    var asset = assetManager.Add(contentAssetData);
                    if (asset != null && asset.Id > 0)
                    {
                        //return the link as well as the status as successful
                        multiupload.folderPath = string.Format("/WorkArea/content.aspx?action=View&folder_id={0}&id={1}&LangType={2}&callerpage=content.aspx", new object[] { asset.FolderId, asset.Id, asset.LanguageId });
                        uploadSuccess = true;
                    }
                }
                else
                {
                    //uploading library items
                    long folderId = long.Parse(context.Request.Form["fid"]);
                    string mime = System.Web.MimeMapping.GetMimeMapping(fileCollection[i].FileName);
                    Ektron.Cms.API.Library libraryAPI = new Ektron.Cms.API.Library();
                    Ektron.Cms.LibraryConfigData lib_setting_data = libraryAPI.GetLibrarySettings(folderId);
                    Ektron.Cms.Framework.Content.LibraryManager libraryManager = new Ektron.Cms.Framework.Content.LibraryManager();
                    //check the mime types on the server to see if the file is an image or file
                    bool image = mime.ToString().ToLower().Contains("image") ? true : false;
                    //populate library data
                    Ektron.Cms.LibraryData libData = new Ektron.Cms.LibraryData()
                    {
                        ParentId = folderId,
                        Title = fileCollection[i].FileName.ToString().Substring(0, fileCollection[i].FileName.IndexOf('.')),
                        Teaser = "",
                        File = fileData,
                        LibraryType = image ? Ektron.Cms.LibraryType.images : Ektron.Cms.LibraryType.files,
                        ContentType = 7,
                        LanguageId = libraryManager.RequestInformation.ContentLanguage,
                        FileName = image ? (lib_setting_data.ImageDirectory + System.IO.Path.GetFileName(fileCollection[i].FileName)).ToLower() : (lib_setting_data.FileDirectory + System.IO.Path.GetFileName(fileCollection[i].FileName)).ToLower()
                    };
                    //add library item
                    var asset = libraryManager.Add(libData);
                    if (asset != null && asset.Id > 0)
                    {
                        //return the link as well as the status as successful
                        multiupload.folderPath = String.Format("/WorkArea/library.aspx?LangType={0}&action=ViewLibraryItem&id={1}&parent_id={2}", new object[] { asset.LanguageId, asset.Id, asset.ParentId });
                        uploadSuccess = true;
                    }
                }
                if (uploadSuccess)
                {
                    multiupload.success = true;
                    status += "File: " + fileCollection[i].FileName + " was uploaded successfully";
                }
                else
                {
                    status += "File: " + fileCollection[i].FileName + " was not uploaded successfully";
                }
            }
            catch (Exception ex)
            {
                status += "File: " + fileCollection[i].FileName + " was not uploaded successfully. Please have your network admin view the event viewer on the webserver for details";
                System.Diagnostics.EventLog.WriteEntry("CMS400", status.ToString() + "\n" + ex.Message, System.Diagnostics.EventLogEntryType.Error);
            }
        }
        multiupload.Message = status;
        return multiupload;
    }

    /// <summary>
    /// Check permissions for the ability to add dms/library items
    /// </summary>
    /// <param name="context">current http context</param>
    /// <param name="dms">check dms if true, library if false</param>
    /// <returns>true if allowed</returns>
    private bool PermissionsCheck(HttpContext context, bool dms = true)
    {
        //check permissions directly in the db to make sure the user is logged in first and is a CMS user
        //if (Ektron.Cms.ObjectFactory.GetUser().IsCmsLoggedIn && Ektron.Cms.ObjectFactory.GetUser().IsCmsUser)
        if (Ektron.Cms.ObjectFactory.GetUser().IsLoggedIn && Ektron.Cms.ObjectFactory.GetUser().IsCmsUser)
        {
            //new instance of the permission manager
            Ektron.Cms.Framework.Settings.PermissionManager permissionManager = new Ektron.Cms.Framework.Settings.PermissionManager();
            long folderID = long.Parse(context.Request.Form["fid"]);
            //retrieve permissions for the folder
            var permissions = permissionManager.GetUserPermissionForFolder(permissionManager.RequestInformation.UserId, folderID, permissionManager.RequestInformation.ContentLanguage);
            if (dms)
            {
                //check add permissions for dms documents
                if (permissions.CanAdd)
                {
                    return true;
                }
            }
            else
            {
                //check add permissions for library items
                if (permissions.CanAddToFileLib && permissions.CanAddToImageLib)
                {
                    return true;
                }
            }
        }
        return false;
    }
}

/// <summary>
/// DMSMultiUpload class to return the json object needed for dropzonejs
/// </summary>
public class DMSMultiUpload
{
    //success = true, fail = false
    public bool success { get; set; }
    //message to return to the user
    public string Message { get; set; }
    //path of the item uploaded (in the workarea)
    public string folderPath { get; set; }
}